import pandas as pd
import numpy as np
from matchms import Spectrum
from matchms.similarity import CosineGreedy
from matplotlib import pyplot as plt


from json import loads as loads
from requests import get


def gnps_id_to_spec(gnps_id):
    usi = f'mzspec:GNPS:GNPS-LIBRARY:accession:{gnps_id}'
    # get spectrum from USI
    url = 'https://metabolomics-usi.gnps2.org/json/?usi1=' + usi
    response = get(url)
    json_data = loads(response.text)
    ms2_mz = np.array(json_data['peaks'])[:, 0]
    ms2_int = np.array(json_data['peaks'])[:, 1]

    return ms2_mz, ms2_int, json_data['precursor_mz']


def get_gnps_std_spec():
    gnps_id_ls = ['CCMSLIB00010011135', 'CCMSLIB00010012730', 'CCMSLIB00010011156']
    names = ['Citrulline-C9:0', '3-hydroxy lauric acid-C22:5', 'L-DOPA-C10:0']

    spectrum_ls = []
    for gnps_id, name in zip(gnps_id_ls, names):
        ms2_mz, ms2_int, prec_mz = gnps_id_to_spec(gnps_id)

        # 1% relative intensity
        mask = (ms2_int > max(ms2_int) * 0.01) & (ms2_mz < prec_mz - 0.1)
        ms2_mz = ms2_mz[mask]
        ms2_int = ms2_int[mask]
        spectrum = Spectrum(mz=ms2_mz, intensities=ms2_int,
                            metadata={"precursor_mz": prec_mz,
                                      "name": name})
        spectrum_ls.append(spectrum)

    return spectrum_ls


def get_std_pms2():
    df = pd.read_csv(
        '/bin/ms1id/lcms/analysis/std/output_all/mixture_Drugs_0eV_feature_table.tsv',
        sep='\t', low_memory=False)

    df = df[(df['MS1_precursor_type'].isin(['[M+H]+'])) & (df['MS1_matched_peak'] > 4)]
    # df = df[(df['MS1_precursor_type'].isin(['[M+H]+', '[M+H-H2O]+'])) & (df['MS1_matched_peak'] > 4)]

    #  sort by MS1_spectral_usage
    df = df.sort_values('MS1_matched_peak', ascending=False)

    # take first 10 rows
    df = df.iloc[:10]

    spectrum_ls = []
    for i, row in df.iterrows():

        pms2 = row['pseudo_ms2'].split(';')
        pms2 = [x.strip().split(' ') for x in pms2][:-1]
        pms2 = [[float(x) for x in y] for y in pms2]
        pms2 = np.array(pms2, dtype=np.float32)

        # remove ions larger than precursor
        pms2 = pms2[pms2[:, 0] < row['m/z'] - 0.1]

        mzs = pms2[:, 0]
        intensities = pms2[:, 1]

        # sort
        idx = np.argsort(mzs)
        mzs = mzs[idx]
        intensities = intensities[idx]

        spectrum = Spectrum(mz=mzs, intensities=intensities,
                            metadata={"precursor_mz": float(row['m/z']),
                                      "name": row['MS1_annotation']})
        spectrum_ls.append(spectrum)

    return spectrum_ls


def get_ref_pms2():
    df = pd.read_csv(
        '/data/nist/fullscan_0ev/aligned_feature_table.tsv',
        sep='\t', low_memory=False)

    df = df[(df['MS1_similarity'].notnull()) & (df['MS1_matched_peak'] > 4)]

    #  sort by MS1_spectral_usage
    df = df.sort_values('MS1_matched_peak', ascending=False)

    # # take first 3 rows
    # df = df.iloc[:3]

    spectrum_ls = []
    for i, row in df.iterrows():

        pms2 = row['pseudo_ms2'].split(';')
        pms2 = [x.strip().split(' ') for x in pms2][:-1]
        pms2 = [[float(x) for x in y] for y in pms2]
        pms2 = np.array(pms2, dtype=np.float32)

        # remove ions larger than precursor
        pms2 = pms2[pms2[:, 0] < row['m/z'] + 0.1]

        mzs = pms2[:, 0]
        intensities = pms2[:, 1]

        # sort
        idx = np.argsort(mzs)
        mzs = mzs[idx]
        intensities = intensities[idx]

        spectrum = Spectrum(mz=mzs, intensities=intensities,
                            metadata={"precursor_mz": float(row['m/z']),
                                      "name": row['MS1_annotation']})
        spectrum_ls.append(spectrum)

    return spectrum_ls


def main(spectrum_ls):

    df = pd.read_csv(
        '/data/PR000639/output/hilic_pos_aligned_feature_table.tsv',
        sep='\t', low_memory=False)

    # Use factory to construct a similarity function
    cosine_greedy = CosineGreedy(tolerance=0.02)

    for i, spec in enumerate(spectrum_ls):
        prec_mz = spec.get("precursor_mz")
        name = spec.get("compound_name")

        sub_df = df[(df['m/z'] > prec_mz - 0.01) & (df['m/z'] < prec_mz + 0.01) & pd.notnull(df['pseudo_ms2'])]

        if len(sub_df) == 0:
            continue

        for j, row in sub_df.iterrows():
            pms2 = row['pseudo_ms2'].split(';')
            pms2 = [x.split(' ') for x in pms2][:-1]
            pms2 = [[float(x) for x in y] for y in pms2]
            pms2 = np.array(pms2, dtype=np.float32)

            # remove ions larger than precursor
            pms2 = pms2[pms2[:, 0] < prec_mz + 0.1]

            mzs = pms2[:, 0]
            intensities = pms2[:, 1]

            # sort
            idx = np.argsort(mzs)
            mzs = mzs[idx]
            intensities = intensities[idx]

            qry_spec = Spectrum(mz=mzs, intensities=intensities,
                                metadata={"precursor_mz": float(row['m/z'])})

            score = cosine_greedy.pair(spec, qry_spec)

            if score['matches'] > 5 and score['score'] > 0.5:
                spec.plot_against(qry_spec)
                plt.xlim(0, float(row['m/z']) * 1.2)
                plt.show()
                print(f"{name} matched with ID {row['ID']} with score {score['score']:.2f} and {score['matches']} matched peaks")


if __name__ == "__main__":
    spectrum_ls = get_ref_pms2()
    main(spectrum_ls)



import os
import pickle
import pandas as pd
from tqdm import tqdm
import requests
from collections import Counter
from rdkit import Chem
from myopic_mces import MCES


def remove_stereochemistry(smiles):
    if pd.isnull(smiles):
        return smiles

    mol = Chem.MolFromSmiles(smiles)

    if mol is None:
        return None  # Invalid SMILES

    Chem.RemoveStereochemistry(mol)
    new_smiles = Chem.MolToSmiles(mol)

    return new_smiles


def calculate_mces(smiles1, smiles2):
    try:
        mces_result = MCES(smiles1, smiles2, threshold=5, always_stronger_bound=False)
        return mces_result[1]
    except:
        return None


def inchikey_to_smiles(inchikey):
    """
    Convert an InChIKey to SMILES using the PubChem PUG REST API.
    If the original InChIKey isn't found, it tries with a modified version. -UHFFFAOYSA-N
    """
    base_url = "https://pubchem.ncbi.nlm.nih.gov/rest/pug"

    def fetch_cid(key):
        cid_url = f"{base_url}/compound/inchikey/{key}/cids/JSON"
        response = requests.get(cid_url)
        if response.status_code != 200:
            return None
        data = response.json()
        return data.get("IdentifierList", {}).get("CID", [None])[0]

    def fetch_smiles(cid):
        smiles_url = f"{base_url}/compound/cid/{cid}/property/IsomericSMILES/JSON"
        response = requests.get(smiles_url)
        if response.status_code != 200:
            return None
        data = response.json()
        return data.get("PropertyTable", {}).get("Properties", [{}])[0].get("IsomericSMILES")

    # Try with original InChIKey
    cid = fetch_cid(inchikey)

    if cid is None:
        # If not found, modify InChIKey and try again
        modified_inchikey = inchikey[:14] + "-UHFFFAOYSA-N"
        print(f"Original InChIKey not found. Trying modified InChIKey: {modified_inchikey}")
        cid = fetch_cid(modified_inchikey)

    if cid is None:
        print(f"No CID found for InChIKey {inchikey} or its modified version")
        return None

    smiles = fetch_smiles(cid)
    if smiles is None:
        print(f"No SMILES found for CID {cid}")
        return None

    return smiles


def gen_all_smiles():
    df1 = pd.read_csv('k0_results.tsv', sep='\t')
    df2 = pd.read_csv('k10_results.tsv', sep='\t')
    df = pd.concat([df1, df2]).reset_index(drop=True)

    # dict from qry_inchikey_14 to qry_smiles
    dict_1 = dict(zip(df['qry_inchikey_14'], df['qry_smiles']))
    df1['matched_smiles'] = df1['matched_inchikey_14'].map(dict_1, na_action='ignore')  # update df1
    df2['matched_smiles'] = df2['matched_inchikey_14'].map(dict_1, na_action='ignore')

    # rest inchikey_14 without smiles
    remain_qry_inchikey_14 = set(df1[df1['qry_smiles'].isnull()]['qry_inchikey_14']) | set(
        df2[df2['qry_smiles'].isnull()]['qry_inchikey_14'])
    remain_matched_inchikey_14 = set(df1[df1['matched_smiles'].isnull()]['matched_inchikey_14']) | set(
        df2[df2['matched_smiles'].isnull()]['matched_inchikey_14'])

    all_inchikeys = list(remain_qry_inchikey_14 | remain_matched_inchikey_14)
    # remove nan
    all_inchikeys = [x for x in all_inchikeys if pd.notnull(x)]

    print(f"Total unique InChIKeys w/o SMILES: {len(all_inchikeys)}")

    ########################################################
    cmpd_metadata = pd.read_parquet('/Users/shipei/Documents/resources/cmpd_properties/20230806_merged_metadata_drug_Nina_Corinna_DOA_GNPS.parquet',
                                    engine='pyarrow')
    cmpd_metadata = cmpd_metadata[['split_inchikey', 'canonical_smiles']]
    # remove rows with nan, using pd.notnull
    cmpd_metadata = cmpd_metadata[pd.notnull(cmpd_metadata['split_inchikey']) & pd.notnull(cmpd_metadata['canonical_smiles'])].reset_index(drop=True)
    # drop duplicates
    cmpd_metadata = cmpd_metadata.drop_duplicates('split_inchikey').reset_index(drop=True)
    # dict from split_inchikey to canonical_smiles
    cmpd_metadata_dict = dict(zip(cmpd_metadata['split_inchikey'], cmpd_metadata['canonical_smiles']))

    ########################################################
    # Convert InChIKeys to SMILES, using GNPS DB
    # load gnps db metadata df
    gnps = pd.read_pickle('/Users/shipei/Documents/projects/ms1_id/data/ms2db/gnps.pkl')
    gnps = gnps[['inchikey_14', 'smiles']]
    # remove nan rows
    gnps = gnps[(gnps['inchikey_14'].notnull()) & (gnps['smiles'].notnull()) & (gnps['smiles'] != 'N/A')].reset_index(
        drop=True)
    # Group by inchikey_14 and find the most common SMILES for each
    gnps_grouped = gnps.groupby('inchikey_14')['smiles'].agg(lambda x: Counter(x).most_common(1)[0][0]).reset_index()
    # Convert to dictionary for faster lookup
    gnps_dict = dict(zip(gnps_grouped['inchikey_14'], gnps_grouped['smiles']))

    ########################################################
    # create a dictionary to store InChIKey to SMILES
    inchikey_to_smiles_dict = {}
    for inchikey in tqdm(all_inchikeys):
        if inchikey in cmpd_metadata_dict:
            inchikey_to_smiles_dict[inchikey] = cmpd_metadata_dict[inchikey]
        elif inchikey in gnps_dict:
            inchikey_to_smiles_dict[inchikey] = gnps_dict[inchikey]
        else:
            continue

    # save, inchikeys are first 14 characters
    pickle.dump(inchikey_to_smiles_dict, open('inchikey_to_smiles_dict.pkl', 'wb'))

    ########################################################
    # update df1 and df2 with smiles
    df1.loc[df1['qry_smiles'].isnull(), 'qry_smiles'] = df1.loc[df1['qry_smiles'].isnull(), 'qry_inchikey_14'].map(inchikey_to_smiles_dict)
    df1.loc[df1['matched_smiles'].isnull(), 'matched_smiles'] = df1.loc[df1['matched_smiles'].isnull(), 'matched_inchikey_14'].map(inchikey_to_smiles_dict)

    df2.loc[df2['qry_smiles'].isnull(), 'qry_smiles'] = df2.loc[df2['qry_smiles'].isnull(), 'qry_inchikey_14'].map(inchikey_to_smiles_dict)
    df2.loc[df2['matched_smiles'].isnull(), 'matched_smiles'] = df2.loc[df2['matched_smiles'].isnull(), 'matched_inchikey_14'].map(inchikey_to_smiles_dict)

    ########################################################
    # remove stereochemistry
    df1['qry_smiles'] = df1['qry_smiles'].apply(remove_stereochemistry)
    df1['matched_smiles'] = df1['matched_smiles'].apply(remove_stereochemistry)

    df1.to_csv('k0_results_smiles.tsv', sep='\t', index=False)
    df2.to_csv('k10_results_smiles.tsv', sep='\t', index=False)


def calc_mces_for_all(mode='k0'):
    # Read the CSV file
    df = pd.read_csv(f'{mode}_results_smiles.tsv', sep='\t')

    # Initialize mces_dist column with None
    df['mces_dist'] = None

    # Fill rows with the same inchikey_14 with 0
    df.loc[df['qry_inchikey_14'] == df['matched_inchikey_14'], 'mces_dist'] = 0

    # Get unique pairs of SMILES for different inchikey_14
    unique_pairs = df[
        (df['mces_dist'].isna()) &
        (pd.notnull(df['qry_smiles'])) &
        (pd.notnull(df['matched_smiles']))
        ][['qry_smiles', 'matched_smiles']].drop_duplicates()

    # Calculate MCES for unique pairs
    if os.path.exists('mces_dict.pkl'):
        mces_dict = pickle.load(open('mces_dict.pkl', 'rb'))
    else:
        mces_dict = {}

    for _, row in tqdm(unique_pairs.iterrows(), total=len(unique_pairs)):

        if (row['qry_smiles'], row['matched_smiles']) in mces_dict:
            continue

        mces = calculate_mces(row['qry_smiles'], row['matched_smiles'])
        mces_dict[(row['qry_smiles'], row['matched_smiles'])] = mces

    # Save the MCES dictionary
    pickle.dump(mces_dict, open('mces_dict.pkl', 'wb'))

    # Fill back the MCES values
    df.loc[df['mces_dist'].isna(), 'mces_dist'] = df[df['mces_dist'].isna()].apply(
        lambda row: mces_dict.get((row['qry_smiles'], row['matched_smiles'])), axis=1
    )

    # Save the results
    df.to_csv(f'{mode}_results_mces.tsv', sep='\t', index=False)


if __name__ == '__main__':

    gen_all_smiles()

    calc_mces_for_all('k0')

    calc_mces_for_all('k10')



import pandas as pd
import numpy as np
import matplotlib.pyplot as plt
from json import loads as loads
from requests import get

def usi_to_spec(usi):
    # get spectrum from USI
    url = 'https://metabolomics-usi.gnps2.org/json/?usi1=' + usi
    response = get(url)
    json_data = loads(response.text)
    ms2_mz = np.array(json_data['peaks'])[:, 0]
    ms2_int = np.array(json_data['peaks'])[:, 1]

    return ms2_mz, ms2_int, json_data['precursor_mz']


def gnps_id_to_spec(gnps_id):
    usi = f'mzspec:GNPS:GNPS-LIBRARY:accession:{gnps_id}'
    return usi_to_spec(usi)


def create_mirror_plot(mz_arr, int_arr, pre_mz,
                       mz_arr_ref, int_arr_ref, fig_size=(2.2, 1.0),
                       ms2_tol=0.05, peak_int_power=0.5,
                       intensity_threshold_label=0, label_decimals=2,
                       mz_range=(100, 1000),
                       max_x=None, plot_unmatched_qry=True,
                       matched_peak_color='red',
                       up_spec_peak_color='0.6', down_spec_peak_color='0.2',
                       save=False,
                       output_name='demo.svg'):
    plt.figure(figsize=fig_size)

    int_arr = np.power(int_arr, peak_int_power)
    int_arr_ref = np.power(int_arr_ref, peak_int_power)

    # mz range
    mask = (mz_arr >= mz_range[0]) & (mz_arr <= mz_range[1])
    mz_arr = mz_arr[mask]
    int_arr = int_arr[mask]
    mask = (mz_arr_ref >= mz_range[0]) & (mz_arr_ref <= mz_range[1])
    mz_arr_ref = mz_arr_ref[mask]
    int_arr_ref = int_arr_ref[mask]

    # Find matched peaks
    matched_indices = []
    matched_indices_ref = []
    for i, mz in enumerate(mz_arr):
        matches = np.where(np.abs(mz_arr_ref - mz) <= ms2_tol)[0]
        if matches.size > 0:
            print(f'{mz:.4f} matched to {mz_arr_ref[matches]}')
            matched_indices.extend([i] * len(matches))
            matched_indices_ref.extend(matches)

    # Normalize intensities
    max_intensity = max(int_arr[matched_indices])
    int_arr = int_arr / max_intensity * 100

    max_intensity_ref = max(int_arr_ref[mz_arr_ref < (pre_mz - 0.5)])
    int_arr_ref = int_arr_ref / max_intensity_ref * 100

    # Set axis limits and labels
    if max_x is None:
        max_x = max(np.max(mz_arr), np.max(mz_arr_ref)) * 1.05  # Add 5% margin

    # Plot unmatched peaks
    if plot_unmatched_qry:
        plt.vlines(mz_arr, 0, int_arr, color=up_spec_peak_color, linewidth=1.20)
    plt.vlines(mz_arr_ref, 0, -int_arr_ref, color=down_spec_peak_color, linewidth=1.20)

    # Plot matched peaks
    plt.vlines(mz_arr[matched_indices], 0, int_arr[matched_indices], color=matched_peak_color, linewidth=1.20)
    plt.vlines(mz_arr_ref[matched_indices_ref], 0, -int_arr_ref[matched_indices_ref], color=matched_peak_color,
               linewidth=1.20)

    plt.xlim(0, max_x)
    plt.ylim(-105, 105)

    # Add labels for peaks above threshold
    for mz, intensity in zip(mz_arr, int_arr):
        if intensity >= intensity_threshold_label and mz < max_x:
            plt.text(mz, intensity + 0.1, f'{mz:.{label_decimals}f}', ha='center', va='bottom', fontsize=8,
                     fontname='Arial')

    for mz, intensity in zip(mz_arr_ref, int_arr_ref):
        if intensity >= intensity_threshold_label and mz < max_x:
            plt.text(mz, -intensity - 2, f'{mz:.{label_decimals}f}', ha='center', va='top', fontsize=8,
                     fontname='Arial')

    # plt.xlabel(r'$\mathit{m/z}$', fontname='Arial', fontsize=12, labelpad=2, color='0.2')
    plt.ylabel('Intensity (%)', fontname='Arial', fontsize=9, labelpad=1, color='0.2')

    plt.gca().xaxis.set_visible(False)

    # Remove top and right spines
    plt.gca().spines['top'].set_visible(False)
    plt.gca().spines['bottom'].set_visible(False)
    plt.gca().spines['right'].set_visible(False)
    # Set the color of the frame (axes spines)
    for spine in plt.gca().spines.values():
        spine.set_color('0.5')

    # Remove grid
    plt.grid(False)

    # Add line at y=0
    plt.axhline(y=0, color='0.5', linewidth=1)

    # Use Helvetica font for tick labels
    # plt.xticks(fontname='Arial', fontsize=10)
    plt.yticks(fontname='Arial', fontsize=8)

    # Adjust tick parameters
    # plt.tick_params(axis='x', pad=4, length=2, color='0.5')
    plt.tick_params(axis='y', pad=0.5, length=1.5, color='0.5')

    # Set the color of tick labels to 0.2
    plt.gca().tick_params(axis='both', colors='0.2')

    # Modify y-axis ticks and labels
    yticks = plt.gca().get_yticks()
    yticks = [y for y in yticks if abs(y) <= 100]
    plt.gca().set_yticks(yticks)
    plt.gca().set_yticklabels([str(abs(int(y))) for y in yticks])

    # Save the plot
    plt.tight_layout()
    if save:
        plt.savefig(output_name, transparent=True)
    plt.show()
    plt.close()


def demo(name, scale):
    df = pd.read_csv(
        '/imaging/mouse_body/wb xenograft in situ metabolomics test - rms_corrected/ms1_id_annotations_derep.tsv',
        sep='\t', low_memory=False)

    df['gnps_id'] = df['db_id'].apply(lambda x: x.split('=')[1].split(';')[0])

    idx = df[(df['name'] == name)].index[0]

    print(df.iloc[idx]['name'])
    print(df.iloc[idx]['precursor_mz'])
    print(df.iloc[idx]['matched_score'])
    print(df.iloc[idx]['matched_peak'])
    print(df.iloc[idx]['spectral_usage'])

    ms1 = df.iloc[idx]['pseudo_ms2']
    mz_arr = []
    int_arr = []
    for p in ms1.split(';'):
        p = p.strip()
        if not p:
            continue
        mz, inten = p.split(' ')
        mz_arr.append(float(mz))
        int_arr.append(float(inten))
    mz_arr = np.array(mz_arr)
    int_arr = np.array(int_arr)

    mz_arr_ref, int_arr_ref, _ = gnps_id_to_spec(df.iloc[idx]['gnps_id'])

    mask = (int_arr > max(int_arr) * 0.01) & (mz_arr < df.iloc[idx]['precursor_mz'] + 0.5)
    mz_arr = mz_arr[mask]
    int_arr = int_arr[mask]

    mask = (int_arr_ref > max(int_arr_ref) * 0.02) & (mz_arr_ref < df.iloc[idx]['precursor_mz'] + 0.5)
    mz_arr_ref = mz_arr_ref[mask]
    int_arr_ref = int_arr_ref[mask]

    # scale
    if scale:
        factor = mz_arr_ref / df.iloc[idx]['precursor_mz'] * 10
        int_arr_ref = int_arr_ref * np.exp(factor)

    create_mirror_plot(np.array(mz_arr), np.array(int_arr), df.iloc[idx]['precursor_mz'],
                       np.array(mz_arr_ref), np.array(int_arr_ref),
                       ms2_tol=0.05, peak_int_power=0.5,
                       intensity_threshold_label=1e5,
                       label_decimals=2,
                       plot_unmatched_qry=False,
                       mz_range=(150, 1000),
                       max_x=float(df.iloc[idx]['precursor_mz']) + 25,
                       matched_peak_color='#F08976',
                       up_spec_peak_color='0.9', down_spec_peak_color='0.9',
                       fig_size=(2.6, 1),
                       save=True,
                       output_name='carnosine.svg')


if __name__ == '__main__':
    # carnosine
    demo('L-CARNOSINE - 20.0 eV', scale=False)



import pickle
import pandas as pd
import matplotlib.pyplot as plt
from upsetplot import UpSet
import matplotlib.font_manager as fm

# Load the InChIKey sets
hilic_pos = set(pickle.load(open('hilic_pos_inchikey.pkl', 'rb')))
hilic_neg = set(pickle.load(open('hilic_neg_inchikey.pkl', 'rb')))
c8_pos = set(pickle.load(open('c8_pos_inchikey.pkl', 'rb')))
c18_neg = set(pickle.load(open('c18_neg_inchikey.pkl', 'rb')))

# Create a dictionary of sets
data = {
    'HILIC+': hilic_pos,
    'HILIC-': hilic_neg,
    'C8+': c8_pos,
    'C18-': c18_neg
}

# Get all unique InChIKeys as a list
all_inchikeys = list(set.union(*data.values()))

# Create a DataFrame with boolean indicators for each set
df = pd.DataFrame({
    set_name: [inchikey in inchikeys for inchikey in all_inchikeys]
    for set_name, inchikeys in data.items()
}, index=all_inchikeys)

# Count the occurrences of each combination
combinations = df.groupby(list(data.keys())).size()

# Set up fonts
plt.rcParams['font.family'] = 'Arial'

# Font sizes (increased)
TITLE_SIZE = 20
LABEL_SIZE = 24
TICK_SIZE = 14
MODE_NAME_SIZE = 16

# Colors
colors = ['#56648a', '#6280a5', '#8ca5c0', '#8d7e95', '#ca9a96', '#facaa9']

# Create the UpSet plot
fig = plt.figure(figsize=(4, 2))
upset = UpSet(combinations,
              sort_by='cardinality',
              min_subset_size=0,
              show_counts=True,
              orientation='horizontal',
              facecolor=colors[0],
              intersection_plot_elements=10,
              element_size=60)

upset.plot()

# Adjust font sizes
plt.ylabel('Annotated compounds', fontsize=LABEL_SIZE, labelpad=10)

# Adjust tick label sizes
for ax in fig.get_axes():
    ax.tick_params(axis='both', which='major', labelsize=TICK_SIZE)
    for text in ax.texts:
        text.set_fontsize(TICK_SIZE)

# Increase font size for mode names
upset.style_subsets(facecolor='#56648a', edgecolor='black', linewidth=10)

# Manually set the font size for subset labels
for text in plt.gca().texts:
    if text.get_text() in data.keys():
        text.set_fontsize(MODE_NAME_SIZE)

# Adjust the layout
plt.tight_layout()
plt.subplots_adjust(left=0.05, right=0.95, top=0.95, bottom=0.05)

# Save the figure
plt.savefig('inchikey_upset_plot.svg', bbox_inches='tight', transparent=True)
plt.show()


import multiprocessing
import os
import pickle
from collections import defaultdict

import numpy as np
import pyimzml.ImzMLParser as imzml
from numba import njit
from tqdm import tqdm

from ms1_id.msi.centroid_data import centroid_spectrum


def process_ms_imaging_data(imzml_file, ibd_file, mass_detect_int_tol=None,
                            mz_bin_size=0.005,
                            noise_detection='moving_average', sn_factor=5.0,
                            centroided=False,
                            n_processes=None,
                            save=False, save_dir=None):
    validate_inputs(noise_detection)

    parser = imzml.ImzMLParser(imzml_file)

    # Check if results already exist
    if save_dir and check_existing_results(save_dir):
        return load_existing_results(save_dir)

    # Auto-detect intensity threshold if not provided
    if mass_detect_int_tol is None:
        print(f'Auto-denoising MS spectra using {noise_detection} method.')
        if noise_detection == 'percentile':
            mass_detect_int_tol = detect_threshold_percentile(parser, sn_factor)

    mz_intensity_dict, coordinates = process_spectra(parser, noise_detection, mass_detect_int_tol,
                                                     mz_bin_size, sn_factor, centroided, n_processes)

    mz_values, intensity_matrix = convert_to_arrays(mz_intensity_dict, coordinates)

    # Save results if requested
    if save and save_dir:
        print(f'Saving mz values, intensity matrix, and coordinates...')
        save_results(save_dir, mz_values, intensity_matrix, coordinates)

    return mz_values, intensity_matrix, coordinates, parser.polarity


def validate_inputs(noise_detection):
    assert noise_detection in ['percentile', 'moving_average'], \
        "Noise reduction method must be 'percentile' or 'moving_average'"


def check_existing_results(save_dir):
    mz_values_path = os.path.join(save_dir, 'mz_values.npy')
    intensity_matrix_path = os.path.join(save_dir, 'intensity_matrix.npy')
    coordinates_path = os.path.join(save_dir, 'coordinates.pkl')
    return all(os.path.exists(path) for path in [mz_values_path, intensity_matrix_path, coordinates_path])


def load_existing_results(save_dir):
    mz_values = np.load(os.path.join(save_dir, 'mz_values.npy'))
    intensity_matrix = np.load(os.path.join(save_dir, 'intensity_matrix.npy'))
    with open(os.path.join(save_dir, 'coordinates.pkl'), 'rb') as f:
        coordinates = pickle.load(f)
    return mz_values, intensity_matrix, coordinates, None  # Note: parser.polarity is not saved


def detect_threshold_percentile(parser, sn_factor=5.0):
    all_intensities = []
    for idx, _ in enumerate(parser.coordinates):
        _, intensity = parser.getspectrum(idx)
        all_intensities.extend(intensity)

    all_intensities = np.array(all_intensities)
    non_zero_intensities = all_intensities[all_intensities > 0.0]
    threshold = max(np.min(non_zero_intensities) * sn_factor, np.percentile(non_zero_intensities, 10))
    print('Auto-detected intensity threshold (percentile method):', threshold)
    return threshold


def process_spectra(parser, noise_detection, mass_detect_int_tol, mz_bin_size, sn_factor, centroided, n_processes):
    args_list = [(idx, *parser.getspectrum(idx), noise_detection, mass_detect_int_tol,
                  mz_bin_size, sn_factor, centroided)
                 for idx, _ in enumerate(parser.coordinates)]

    n_processes = n_processes or multiprocessing.cpu_count()

    if n_processes == 1:
        # Non-parallel processing
        results = []
        for args in tqdm(args_list, desc="Denoising spectra", unit="spectrum"):
            results.append(process_spectrum(args))
    else:
        # Parallel processing
        with multiprocessing.Pool(processes=n_processes) as pool:
            results = list(tqdm(pool.imap(process_spectrum, args_list),
                                total=len(args_list),
                                desc="Denoising spectra",
                                unit="spectrum"))

    mz_intensity_dict = defaultdict(lambda: defaultdict(float))
    coordinates = []
    for idx, binned_data in results:
        for binned_m, summed_intensity in binned_data.items():
            mz_intensity_dict[binned_m][idx] = summed_intensity
        coordinates.append(parser.coordinates[idx][:2])  # Only x and y

    return mz_intensity_dict, coordinates


def convert_to_arrays(mz_intensity_dict, coordinates):
    mz_values = np.array(sorted(mz_intensity_dict.keys()))
    intensity_matrix = np.array([
        [mz_intensity_dict[mz].get(idx, 0.0) for idx in range(len(coordinates))]
        for mz in mz_values
    ])
    return mz_values, intensity_matrix


def save_results(save_dir, mz_values, intensity_matrix, coordinates):
    np.save(os.path.join(save_dir, 'mz_values.npy'), mz_values)
    np.save(os.path.join(save_dir, 'intensity_matrix.npy'), intensity_matrix)
    with open(os.path.join(save_dir, 'coordinates.pkl'), 'wb') as f:
        pickle.dump(coordinates, f)


def process_spectrum(args):
    idx, mz, intensity, noise_detection, mass_detect_int_tol, mz_bin_size, sn_factor, centroided = args

    if noise_detection == 'moving_average':
        baseline = moving_average_baseline(mz, intensity, factor=sn_factor)
        mask = intensity > baseline
    else:
        mask = intensity > mass_detect_int_tol

    # Filter mz and intensity arrays
    filtered_mz = mz[mask]
    filtered_intensity = intensity[mask]

    # Centroid the spectrum
    if not centroided:
        filtered_mz, filtered_intensity = centroid_spectrum(filtered_mz, filtered_intensity,
                                                            centroid_mode='max',
                                                            width_da=0.005, width_ppm=25)

    # Bin m/z values and take max intensity within each bin
    binned_data = {}
    for m, i in zip(filtered_mz, filtered_intensity):
        binned_m = round(m / mz_bin_size) * mz_bin_size
        if binned_m not in binned_data or i > binned_data[binned_m]:
            binned_data[binned_m] = i

    return idx, binned_data


@njit
def moving_average_baseline(mz_array, intensity_array,
                            mz_window=100.0, percentage_lowest=0.05, factor=5.0):
    """
    Apply moving average algorithm to a single mass spectrum using an m/z-based window.
    This function is optimized with Numba.

    :param mz_array: numpy array of m/z values
    :param intensity_array: numpy array of intensity values
    :param mz_window: size of the moving window in Da (default: 100.0)
    :param percentage_lowest: percentage of lowest points to consider in each window (default: 5%)
    :param factor: factor to multiply the mean of the lowest points (default: 5.0)
    :return: tuple of (filtered_mz_array, filtered_intensity_array)
    """
    baseline_array = []

    for i in range(len(mz_array)):
        mz_min = mz_array[i] - mz_window / 2
        mz_max = mz_array[i] + mz_window / 2

        window_intensities = intensity_array[(mz_array >= mz_min) & (mz_array <= mz_max)]

        if len(window_intensities) > 0:
            positive_intensities = window_intensities[window_intensities > 0]
            if len(positive_intensities) > 0:
                sorted_intensities = np.sort(positive_intensities)
                n_lowest = max(1, int(len(sorted_intensities) * percentage_lowest))
                lowest_n = sorted_intensities[:n_lowest]
                baseline = factor * np.mean(lowest_n)
            else:
                baseline = 0.0
        else:
            baseline = 0.0

        baseline_array.append(baseline)

    return np.array(baseline_array)



import numpy as np
import pandas as pd
import os
import pickle


def merge_ms1_annotations(db='gnps'):
    df_dir = f'../../../../data/nist_samples/data_1/output_{db}'
    files = [f for f in os.listdir(df_dir) if f.endswith('.tsv')]
    files = [os.path.join(df_dir, f) for f in files]

    all_df = pd.DataFrame()
    for f in files:
        df = pd.read_csv(f, sep='\t', low_memory=False)
        df['file'] = f.split('/')[-1].split('.')[0]
        all_df = pd.concat([all_df, df])

    all_df.to_csv(f'merged_ms1_{db}.tsv', sep='\t', index=False)


def merge_ms2_annotations(db='gnps'):
    df_dir = f'../../../../data/nist_samples/data_2/output_{db}'
    files = [f for f in os.listdir(df_dir) if f.endswith('.tsv')]
    files = [os.path.join(df_dir, f) for f in files]

    all_df = pd.DataFrame()
    for f in files:
        df = pd.read_csv(f, sep='\t', low_memory=False)
        df['file'] = f.split('/')[-1].split('.')[0]
        all_df = pd.concat([all_df, df])

    all_df.to_csv(f'merged_ms2_{db}.tsv', sep='\t', index=False)


def ms2_annotation_analysis(db='gnps', score_cutoff=0.7, min_matched_peak=4):
    df_dir = f'../../../../data/nist_samples/data_2/output_{db}'
    files = [f for f in os.listdir(df_dir) if f.endswith('.tsv')]
    files = [os.path.join(df_dir, f) for f in files]

    inchikey_ls = []
    for f in files:
        df = pd.read_csv(f, sep='\t', low_memory=False)

        df = df[df['MS2_similarity'].notnull()]

        df = df[(df['MS2_similarity'] >= score_cutoff) & (df['MS2_matched_peak'] >= min_matched_peak)]

        inchikey_ls += df['MS2_inchikey'].unique().tolist()

    # remove nan
    inchikey_ls = [i for i in inchikey_ls if not pd.isnull(i)]

    inchikey_ls = list(set(inchikey_ls))
    print(f'{db}: {len(inchikey_ls)} unique inchikeys')

    # save
    pickle.dump(inchikey_ls, open(f'ms2_inchikey_{db}.pkl', 'wb'))


def ms1_annotation_analysis(db, score_cutoff=0.7, min_matched_peak=4, ev=0):
    df_dir = f'../../../../data/nist_samples/data_1/output_{db}'
    files = [f for f in os.listdir(df_dir) if f.endswith(f'_{ev}eV_feature_table.tsv')]
    files = [os.path.join(df_dir, f) for f in files]

    inchikey_ls = []
    for f in files:
        df = pd.read_csv(f, sep='\t', low_memory=False)

        df = df[df['MS1_similarity'].notnull()]

        df = df[(df['MS1_similarity'] >= score_cutoff) & (df['MS1_matched_peak'] >= min_matched_peak)]

        inchikey_ls += df['MS1_inchikey'].unique().tolist()

    # remove nan
    inchikey_ls = [i for i in inchikey_ls if not pd.isnull(i)]

    inchikey_ls = list(set(inchikey_ls))
    print(f'{db}: {len(inchikey_ls)} unique inchikeys')

    # save
    pickle.dump(inchikey_ls, open(f'ms1_inchikey_{db}_{ev}ev.pkl', 'wb'))


def annotation_overlap(db='gnps'):
    ms2_inchikey = pickle.load(open(f'ms2_inchikey_{db}.pkl', 'rb'))

    ms1_inchikey_0ev = pickle.load(open(f'ms1_inchikey_{db}_0ev.pkl', 'rb'))

    ms1_inchikey_10ev = pickle.load(open(f'ms1_inchikey_{db}_10ev.pkl', 'rb'))

    # overlap
    print('ms2 and ms1 0 eV')
    ms2_0ev_overlap = set(ms2_inchikey).intersection(set(ms1_inchikey_0ev))
    print(f'{len(ms2_0ev_overlap)} common inchikeys: {ms2_0ev_overlap}')

    print('ms2 and ms1 10 eV')
    ms2_10ev_overlap = set(ms2_inchikey).intersection(set(ms1_inchikey_10ev))
    print(f'{len(ms2_10ev_overlap)} common inchikeys: {ms2_10ev_overlap}')

    print('0 eV and 10 eV')
    ms1_0_10_overlap = set(ms1_inchikey_0ev).intersection(set(ms1_inchikey_10ev))
    print(f'{len(ms1_0_10_overlap)} common inchikeys: {ms1_0_10_overlap}')

    print('ms2 and ms1 0 eV and 10 eV')
    overlap = set(ms2_inchikey).intersection(set(ms1_inchikey_0ev)).intersection(set(ms1_inchikey_10ev))
    print(f'{len(overlap)} common inchikeys: {overlap}')

    print('in ms1 0 eV or 10 eV but not in ms2')
    ms1_not_ms2 = set(ms1_inchikey_0ev).union(set(ms1_inchikey_10ev)).difference(set(ms2_inchikey))
    # save
    pickle.dump(ms1_not_ms2, open(f'ms1_not_ms2_{db}.pkl', 'wb'))


def ms1_not_in_ms2(db):
    ms1_not_ms2 = list(pickle.load(open(f'ms1_not_ms2_{db}.pkl', 'rb')))

    # merged ms1
    ms1_df = pd.read_csv(f'merged_ms1_{db}.tsv', sep='\t', low_memory=False)
    ms1_df = ms1_df[ms1_df['MS1_inchikey'].notnull()].reset_index(drop=True)
    ms1_df = ms1_df[ms1_df['MS1_inchikey'].isin(ms1_not_ms2)]

    # find in feature table for ms2
    ms2_df = pd.read_csv(f'merged_ms2_{db}.tsv', sep='\t', low_memory=False)

    ms1_df['ms2_in_feature_table'] = False
    for i, row in ms1_df.iterrows():
        prec_mz = row['m/z']
        rt = row['RT']
        # find in feature table, precmz tolerance 0.01, rt tolerance 0.05
        df = ms2_df[((ms2_df['m/z'] - prec_mz).abs() <= 0.01) & ((ms2_df['RT'] - rt).abs() <= 0.05)]
        if len(df) > 0:
            # if at least one row has MS2
            if df['MS2'].notnull().any():
                ms1_df.at[i, 'ms2_in_feature_table'] = True

    ms1_df.to_csv(f'ms1_not_ms2_{db}.tsv', sep='\t', index=False)


def ms2_ce_analysis(score_cutoff, min_matched_peak):
    """
    collision energy analysis
    """
    df_dir = '/data/nist_samples/data_2/output_nist20'
    files = [f for f in os.listdir(df_dir) if f.endswith('.tsv')]
    files = [os.path.join(df_dir, f) for f in files]

    all_df = pd.DataFrame()
    for f in files:
        df = pd.read_csv(f, sep='\t', low_memory=False)

        df = df[df['MS2_similarity'].notnull()]

        df = df[(df['MS2_similarity'] >= score_cutoff) & (df['MS2_matched_peak'] >= min_matched_peak)]

        all_df = pd.concat([all_df, df])

    # all_df['ms2_ce_valid'] = all_df['MS2_collision_energy'].apply(lambda x: 'eV' in str(x))
    all_df['ms2_ce_valid'] = all_df['MS2_collision_energy'].apply(lambda x: 'NCE' in str(x))

    # sort by inchikey, then ms2_ce_valid, then MS2_similarity
    all_df = all_df.sort_values(['MS2_inchikey', 'ms2_ce_valid', 'MS2_similarity'],
                                ascending=[True, False, False])
    # remove duplicates
    all_df = all_df.drop_duplicates('MS2_inchikey', keep='first')

    # all_df['ms2_ce'] = all_df.apply(lambda x: x['MS2_collision_energy'].split(' ')[1].split('eV')[0] if x['ms2_ce_valid'] else None, axis=1)
    all_df['ms2_ce'] = all_df.apply(
        lambda x: x['MS2_collision_energy'].split('%')[0].split('=')[1] if x['ms2_ce_valid'] else None, axis=1)

    all_ce = all_df['ms2_ce'].tolist()
    all_ce = [int(c) for c in all_ce if c is not None]

    # save to file
    all_df.to_csv('ms2_ce_analysis.tsv', sep='\t', index=False)
    pickle.dump(all_ce, open('ms2_ce.pkl', 'wb'))


def ms1_ce_analysis(score_cutoff, min_matched_peak, ev=0):
    """
    collision energy analysis
    """
    df_dir = '/data/nist_samples/data_1/output_nist20'
    files = [f for f in os.listdir(df_dir) if f.endswith(f'_{ev}eV_feature_table.tsv')]
    files = [os.path.join(df_dir, f) for f in files]

    all_df = pd.DataFrame()
    for f in files:
        df = pd.read_csv(f, sep='\t', low_memory=False)

        df = df[df['MS1_similarity'].notnull()]

        df = df[(df['MS1_similarity'] >= score_cutoff) & (df['MS1_matched_peak'] >= min_matched_peak)]

        all_df = pd.concat([all_df, df])

    # all_df['ms1_ce_valid'] = all_df['MS1_collision_energy'].apply(lambda x: 'eV' in str(x))
    all_df['ms1_ce_valid'] = all_df['MS1_collision_energy'].apply(lambda x: 'NCE' in str(x))

    # sort by inchikey, then ms1_ce_valid, then MS1_similarity
    all_df = all_df.sort_values(['MS1_inchikey', 'ms1_ce_valid', 'MS1_similarity'],
                                ascending=[True, False, False])
    # remove duplicates
    all_df = all_df.drop_duplicates('MS1_inchikey', keep='first')

    # all_df['ms1_ce'] = all_df.apply(lambda x: x['MS1_collision_energy'].split(' ')[1].split('eV')[0] if x['ms1_ce_valid'] else None, axis=1)
    all_df['ms1_ce'] = all_df.apply(
        lambda x: x['MS1_collision_energy'].split('%')[0].split('=')[1] if x['ms1_ce_valid'] else None, axis=1)

    all_ce = all_df['ms1_ce'].tolist()
    all_ce = [int(c) for c in all_ce if c is not None]

    # save to file
    all_df.to_csv(f'ms1_ce_analysis_{ev}ev.tsv', sep='\t', index=False)
    pickle.dump(all_ce, open(f'ms1_ce_{ev}ev.pkl', 'wb'))


def plot_ms2_ce():
    import matplotlib.pyplot as plt

    all_ce = pickle.load(open('ms2_ce.pkl', 'rb'))

    # Create histogram
    plt.figure(figsize=(5, 3))
    plt.hist(all_ce, bins=np.arange(0, max(all_ce) + 5, 10), color='#56648a', alpha=0.6)
    plt.title('Histogram of MS2 Collision Energies', fontname='Arial', fontsize=12)
    plt.xlabel('NCE (%)')
    plt.ylabel('Frequency')
    plt.xlim(-5, 70)
    plt.savefig('ms2_ce_histogram.svg', transparent=True)
    plt.show()


def plot_ms1_ce(ev=0):
    import matplotlib.pyplot as plt

    all_ce = pickle.load(open(f'ms1_ce_{ev}ev.pkl', 'rb'))

    # Create histogram
    plt.figure(figsize=(5, 3))
    plt.hist(all_ce, bins=np.arange(0, max(all_ce) + 5, 10), color='#56648a', alpha=0.6)
    plt.title('Histogram of MS1 Collision Energies', fontname='Arial', fontsize=12)
    plt.xlabel('NCE (%)')
    plt.ylabel('Frequency')
    plt.xlim(-5, 70)
    # plt.ylim(0, 100)
    plt.savefig(f'ms1_ce_histogram_{ev}ev.svg', transparent=True)
    plt.show()


if __name__ == '__main__':

    score_cutoff = 0.7
    min_matched_peak = 4
    ############################
    ##### annotation analysis #####
    # merge_ms1_annotations(db='gnps')
    # merge_ms1_annotations(db='nist20')
    # merge_ms2_annotations(db='gnps')
    # merge_ms2_annotations(db='nist20')

    # print('=' * 20)
    # print('MS2 annotation')
    # ms2_annotation_analysis('gnps', score_cutoff, min_matched_peak)
    # ms2_annotation_analysis('nist20', score_cutoff, min_matched_peak)
    # #
    # print('=' * 20)
    # print('MS1 annotation')
    # ms1_annotation_analysis('gnps', score_cutoff, min_matched_peak, 0)
    # ms1_annotation_analysis('nist20', score_cutoff, min_matched_peak, 0)
    # #
    # ms1_annotation_analysis('gnps', score_cutoff, min_matched_peak, 10)
    # ms1_annotation_analysis('nist20', score_cutoff, min_matched_peak, 10)
    #
    # print('=' * 20)
    # print('Annotation overlap')
    # annotation_overlap('gnps')
    # annotation_overlap('nist20')
    #
    # ms1_not_in_ms2('gnps')

    ############################
    ##### collision energy analysis #####
    # ms2_ce_analysis(score_cutoff, min_matched_peak)
    # ms1_ce_analysis(score_cutoff, min_matched_peak, 0)
    # ms1_ce_analysis(score_cutoff, min_matched_peak, 10)

    plot_ms2_ce()
    plot_ms1_ce(ev=0)
    plot_ms1_ce(ev=10)



import matplotlib.pyplot as plt
import numpy as np
import pickle
from matplotlib.colors import to_rgba, to_hex


def blend_color(color, alpha, bg_color='#ffffff'):
    """Blend a color with a background color based on alpha."""
    c = np.array(to_rgba(color))
    bg = np.array(to_rgba(bg_color))
    return to_hex((1 - alpha) * bg[:3] + alpha * c[:3])


def plot_combined_ce():
    # Load data
    ms2_ce = pickle.load(open('ms2_ce.pkl', 'rb'))
    ms1_ce_0ev = pickle.load(open('ms1_ce_0ev.pkl', 'rb'))
    ms1_ce_10ev = pickle.load(open('ms1_ce_10ev.pkl', 'rb'))

    # Set up colors
    # colors = ['#56648a', '#6280a5', '#8ca5c0', '#8d7e95', '#ca9a96', '#facaa9']
    base_colors = ['#8d7e95', '#ca9a96', '#56648a']
    alphas = [0.3, 0.7, 0.6]

    # Blend colors
    colors = [blend_color(color, alpha) for color, alpha in zip(base_colors, alphas)]

    # Create plot
    plt.figure(figsize=(4.5, 3))

    # Plot histograms
    plt.hist(ms2_ce, bins=np.arange(0, 75, 10), color=colors[0], label='MS/MS')

    plt.hist(ms1_ce_10ev, bins=np.arange(0, 75, 10), color=colors[2], label='MS1 (10 eV)')
    plt.hist(ms1_ce_0ev, bins=np.arange(0, 75, 10), color=colors[1], label='MS1 (0 eV)')

    # Calculate histograms
    bins = np.arange(0, 75, 10)
    hist_ms2, _ = np.histogram(ms2_ce, bins=bins)
    hist_ms1_0ev, _ = np.histogram(ms1_ce_0ev, bins=bins)
    hist_ms1_10ev, _ = np.histogram(ms1_ce_10ev, bins=bins)

    # Plot upper edges of histograms
    # plt.plot(bins[:-1] + 5, hist_ms2, color=base_colors[0], alpha=1, label='MS/MS', linewidth=1.75)
    # plt.plot(bins[:-1] + 5, hist_ms1_0ev, color=base_colors[1], alpha=1, label='MS1 (0 eV)', linewidth=1.75)
    # plt.plot(bins[:-1] + 5, hist_ms1_10ev, color=base_colors[2], alpha=1, label='MS1 (10 eV)', linewidth=1.75)

    # Plot upper edges of histograms with horizontal and vertical lines
    for i, (hist, color, label) in enumerate(zip([hist_ms2, hist_ms1_0ev, hist_ms1_10ev],
                                                 base_colors,
                                                 ['MS/MS', 'MS1 (0 eV)', 'MS1 (10 eV)'])):
        for j in range(len(bins) - 1):
            plt.plot([bins[j], bins[j + 1]], [hist[j], hist[j]], color=color, linewidth=1.5)
            if j < len(bins) - 2:
                plt.plot([bins[j + 1], bins[j + 1]], [hist[j], hist[j + 1]], color=color, linewidth=1.75)

        # Add a label for the legend
        plt.plot([], [], color=color, linewidth=1.75, label=f'{label} (upper edge)')

    # Set title and labels
    plt.title('Normalized collision energies of annotations', fontname='Arial', fontsize=14, pad=10)
    plt.xlabel('NCE (%)', fontname='Arial', fontsize=12, labelpad=5)
    plt.ylabel('Frequency', fontname='Arial', fontsize=12, labelpad=8)

    # Set limits
    plt.xlim(-2, 70)
    plt.ylim(-3, 65)

    # Adjust frame and ticks
    ax = plt.gca()
    ax.spines['top'].set_visible(False)
    ax.spines['right'].set_visible(False)
    for spine in ax.spines.values():
        spine.set_color('0.4')

    ax.tick_params(axis='both', which='major', labelsize=10, color='0.4', length=2, width=1, pad=4)

    # Set tick label font
    for label in ax.get_xticklabels() + ax.get_yticklabels():
        label.set_fontname('Arial')

    # Add legend for only the histogram bars
    legend_elements = [plt.Rectangle((0, 0), 1, 1, color=colors[i], label=label)
                       for i, label in enumerate(['MS/MS', 'MS1 (0 eV)', 'MS1 (10 eV)'])]
    plt.legend(handles=legend_elements, prop={'family': 'Arial', 'size': 10})

    # Adjust layout and save
    plt.tight_layout()
    plt.savefig('combined_ce_histogram.svg', transparent=True)
    plt.show()


if __name__ == '__main__':
    plot_combined_ce()


import numpy as np
import matplotlib.pyplot as plt
from matplotlib.ticker import ScalarFormatter, FormatStrFormatter

def exp_scaling(normalized_mz, growth_rate=10.0):
    return np.exp(growth_rate * normalized_mz)

# Generate normalized m/z values
normalized_mz = np.linspace(0, 1, 1000)

# Calculate scaling factors
exp_factors = exp_scaling(normalized_mz, 10.0)

# Set the font to Arial
plt.rcParams['font.family'] = 'Arial'

# Create the plot
fig, ax = plt.subplots(figsize=(2.85, 2))
ax.plot(normalized_mz, exp_factors)

# Set italic 'm/z' in x-label
ax.set_xlabel(r'Normalized $\mathit{m/z}$')
ax.set_ylabel('Scaling factor')

# Use scientific notation for y-axis
ax.yaxis.set_major_formatter(ScalarFormatter(useMathText=True))
ax.ticklabel_format(style='sci', axis='y', scilimits=(0,0))

# Adjust tick length and pad
ax.tick_params(axis='both', which='major', length=2.5, pad=3)

# Set frame color
for spine in ax.spines.values():
    spine.set_edgecolor('0.4')

ax.grid(True, alpha=0.3)  # Make the grid lighter

# Adjust layout to show all elements
plt.tight_layout()

# Save as SVG with transparent background
plt.savefig('plot.svg', format='svg', transparent=True)

plt.show()


import os

from ms1_id.msi.ms1id_msi_workflow import ms1id_imaging_workflow


def ms1id_single_file_batch(file_dir, library_path, n_processes=None,
                            mass_detect_int_tol=None,
                            noise_detection='moving_average',
                            sn_factor=5.0,
                            centroided=False,
                            mz_bin_size=0.01,
                            min_overlap=10, min_correlation=0.85, max_cor_depth=1,
                            library_search_mztol=0.01,
                            ms1id_score_cutoff=0.7,
                            ms1id_min_matched_peak=4,
                            ms1id_min_spec_usage=0.10,
                            max_prec_rel_int_in_other_ms2=0.05
                            ):
    files = [f for f in os.listdir(file_dir) if f.lower().endswith('.imzml')]
    files = [os.path.join(file_dir, f) for f in files]

    for file in files:
        ms1id_imaging_workflow(file, library_path, n_processes=n_processes,
                               mass_detect_int_tol=mass_detect_int_tol,
                               noise_detection=noise_detection,
                               sn_factor=sn_factor,
                               centroided=centroided,
                               mz_bin_size=mz_bin_size,
                               min_overlap=min_overlap, min_correlation=min_correlation,
                               max_cor_depth=max_cor_depth,
                               library_search_mztol=library_search_mztol, ms1id_score_cutoff=ms1id_score_cutoff,
                               ms1id_min_spec_usage=ms1id_min_spec_usage,
                               ms1id_min_matched_peak=ms1id_min_matched_peak,
                               max_prec_rel_int_in_other_ms2=max_prec_rel_int_in_other_ms2
                               )

    return



import pandas as pd
import os
import itertools
from typing import List, Tuple


def process_files(base_dir: str) -> pd.DataFrame:
    all_dfs = []
    for k in ['_k0', '_k3', '_k5', '_k8', '_k9', '_k10', '_k13', '_k15', '_k18']:
        folder = os.path.join(base_dir, f'output{k}')
        files = [f for f in os.listdir(folder) if not f.startswith('.') and '_0eV' in f]

        for file in files:
            df = pd.read_csv(os.path.join(folder, file), sep='\t', low_memory=False)
            df = df[~df['MS1_similarity'].isnull()].reset_index(drop=True)
            df = df[(df['MS1_similarity'] >= 0.7) & (df['MS1_matched_peak'] >= 3)].reset_index(drop=True)
            df['library'] = f'std{k}.pkl'
            all_dfs.append(df)

    return pd.concat(all_dfs, ignore_index=True)


def analyze_library_combinations(data: pd.DataFrame, max_combo: int = 3) -> List[Tuple[List[str], int]]:
    library_counts = data['library'].value_counts()
    libraries = library_counts.index.tolist()
    counts = library_counts.values

    all_combinations = []
    k0_library = 'std_k0.pkl'

    for r in range(1, max_combo + 1):
        for combo in itertools.combinations(libraries, r):
            if k0_library in combo:
                count = len(data[data['library'].isin(combo)])
                all_combinations.append((list(combo), count))

    return sorted(all_combinations, key=lambda x: x[1], reverse=True)


def main(base_dir: str):
    # Process all files
    aggregated_data = process_files(base_dir)

    # Perform combination analysis
    top_combinations = analyze_library_combinations(aggregated_data, 2)

    print(f"Total unique annotations: {len(aggregated_data)}")
    print("\nTop 5 library combinations (including k0):")
    for libs, count in top_combinations[:5]:
        print(f"Libraries: {', '.join(libs)}")
        print(f"Annotations covered: {count}")
        print(f"Coverage percentage: {count / len(aggregated_data) * 100:.2f}%")
        print()

    # Additional analysis: unique annotations per library
    unique_per_library = aggregated_data.groupby('library').apply(
        lambda x: len(x[~x.index.isin(aggregated_data[aggregated_data['library'] != x.name].index)]))
    print("Unique annotations per library:")
    print(unique_per_library)


if __name__ == '__main__':
    base_dir = '/bin/ms1id/lcms/analysis/std'
    main(base_dir)


import pickle

import pandas as pd


def print_annotation_stats(score_threshold=0.7, usage_threshold=0.20):
    # DDA MS2
    ms2_dda_df = pd.read_csv('../../../../data/nist/dda_ms2/aligned_feature_table.tsv', sep='\t', low_memory=False)

    # DDA MS1
    ms1_dda_df = pd.read_csv('../../../../data/nist/dda_ms1/aligned_feature_table.tsv', sep='\t', low_memory=False)

    # Full scan MS1
    ms1_0ev_df = pd.read_csv('../../../../data/nist/fullscan_0ev/aligned_feature_table.tsv', sep='\t', low_memory=False)
    ms1_10ev_df = pd.read_csv('../../../../data/nist/fullscan_10ev/aligned_feature_table.tsv', sep='\t',
                              low_memory=False)
    ms1_20ev_df = pd.read_csv('../../../../data/nist/fullscan_20ev/aligned_feature_table.tsv', sep='\t',
                              low_memory=False)

    print('Total features in each mode:')
    print(f'DDA MS2: {ms2_dda_df.shape[0]}')
    print(f'DDA MS1: {ms1_dda_df.shape[0]}')
    print(f'Full scan MS1 0 eV: {ms1_0ev_df.shape[0]}')
    print(f'Full scan MS1 10 eV: {ms1_10ev_df.shape[0]}')
    print(f'Full scan MS1 20 eV: {ms1_20ev_df.shape[0]}')

    print('DDA MS2 collected features:', ms2_dda_df[ms2_dda_df['MS2'].notnull()].shape[0])
    print('DDA MS2 not collected features:', ms2_dda_df[ms2_dda_df['MS2'].isnull()].shape[0])

    ms2_dda_df = ms2_dda_df[ms2_dda_df['similarity'].notnull()].reset_index(drop=True)
    ms1_dda_df = ms1_dda_df[ms1_dda_df['MS1_similarity'].notnull()].reset_index(drop=True)
    ms1_0ev_df = ms1_0ev_df[ms1_0ev_df['MS1_similarity'].notnull()].reset_index(drop=True)
    ms1_10ev_df = ms1_10ev_df[ms1_10ev_df['MS1_similarity'].notnull()].reset_index(drop=True)
    ms1_20ev_df = ms1_20ev_df[ms1_20ev_df['MS1_similarity'].notnull()].reset_index(drop=True)

    ms2_dda_df = ms2_dda_df[ms2_dda_df['similarity'] >= score_threshold].reset_index(drop=True)
    ms1_dda_df = ms1_dda_df[(ms1_dda_df['MS1_similarity'] >= score_threshold) &
                            (ms1_dda_df['MS1_spectral_usage'] >= usage_threshold)].reset_index(drop=True)
    ms1_0ev_df = ms1_0ev_df[(ms1_0ev_df['MS1_similarity'] >= score_threshold) &
                            (ms1_0ev_df['MS1_spectral_usage'] >= usage_threshold)].reset_index(drop=True)
    ms1_10ev_df = ms1_10ev_df[(ms1_10ev_df['MS1_similarity'] >= score_threshold) &
                              (ms1_10ev_df['MS1_spectral_usage'] >= usage_threshold)].reset_index(drop=True)
    ms1_20ev_df = ms1_20ev_df[(ms1_20ev_df['MS1_similarity'] >= score_threshold) &
                              (ms1_20ev_df['MS1_spectral_usage'] >= usage_threshold)].reset_index(drop=True)

    print('Number of annotated features in each mode:')
    print(f'DDA MS2: {ms2_dda_df.shape[0]}')
    print(f'DDA MS1: {ms1_dda_df.shape[0]}')
    print(f'Full scan MS1 0 eV: {ms1_0ev_df.shape[0]}')
    print(f'Full scan MS1 10 eV: {ms1_10ev_df.shape[0]}')
    print(f'Full scan MS1 20 eV: {ms1_20ev_df.shape[0]}')

    ms2_dda_inchikey = ms2_dda_df['InChIKey'].unique().tolist()
    ms1_dda_inchikey = ms1_dda_df['MS1_inchikey'].unique().tolist()
    ms1_0ev_inchikey = ms1_0ev_df['MS1_inchikey'].unique().tolist()
    ms1_10ev_inchikey = ms1_10ev_df['MS1_inchikey'].unique().tolist()
    ms1_20ev_inchikey = ms1_20ev_df['MS1_inchikey'].unique().tolist()

    # remove nan inchikey
    ms2_dda_inchikey = [i for i in ms2_dda_inchikey if not pd.isnull(i)]
    ms1_dda_inchikey = [i for i in ms1_dda_inchikey if not pd.isnull(i)]
    ms1_0ev_inchikey = [i for i in ms1_0ev_inchikey if not pd.isnull(i)]
    ms1_10ev_inchikey = [i for i in ms1_10ev_inchikey if not pd.isnull(i)]
    ms1_20ev_inchikey = [i for i in ms1_20ev_inchikey if not pd.isnull(i)]

    print('Number of unique inchikey (3D) in each mode:')
    print(f'DDA MS2: {len(ms2_dda_inchikey)}')
    print(f'DDA MS1: {len(ms1_dda_inchikey)}')
    print(f'Full scan MS1 0 eV: {len(ms1_0ev_inchikey)}')
    print(f'Full scan MS1 10 eV: {len(ms1_10ev_inchikey)}')
    print(f'Full scan MS1 20 eV: {len(ms1_20ev_inchikey)}')

    # 2D inchikey
    ms2_dda_inchikey_2d = [i[:14] for i in ms2_dda_inchikey]
    ms1_dda_inchikey_2d = [i[:14] for i in ms1_dda_inchikey]
    ms1_0ev_inchikey_2d = [i[:14] for i in ms1_0ev_inchikey]
    ms1_10ev_inchikey_2d = [i[:14] for i in ms1_10ev_inchikey]
    ms1_20ev_inchikey_2d = [i[:14] for i in ms1_20ev_inchikey]

    print('Number of unique inchikey (2D) in each mode:')
    print(f'DDA MS2: {len(set(ms2_dda_inchikey_2d))}')
    print(f'DDA MS1: {len(set(ms1_dda_inchikey_2d))}')
    print(f'Full scan MS1 0 eV: {len(set(ms1_0ev_inchikey_2d))}')
    print(f'Full scan MS1 10 eV: {len(set(ms1_10ev_inchikey_2d))}')
    print(f'Full scan MS1 20 eV: {len(set(ms1_20ev_inchikey_2d))}')

    # # save
    # pickle.dump(ms2_dda_inchikey_2d, open('data/ms2_dda_inchikey.pkl', 'wb'))
    # pickle.dump(ms1_dda_inchikey_2d, open('data/ms1_dda_inchikey.pkl', 'wb'))
    # pickle.dump(ms1_0ev_inchikey_2d, open('data/ms1_0ev_inchikey.pkl', 'wb'))
    # pickle.dump(ms1_10ev_inchikey_2d, open('data/ms1_10ev_inchikey.pkl', 'wb'))
    # pickle.dump(ms1_20ev_inchikey_2d, open('data/ms1_20ev_inchikey.pkl', 'wb'))


def align_feature_tables(score_threshold=0.7, usage_threshold=0.20):
    """
    based on feature table from DDA MS2 analysis, align results from DDA MS1, Full scan MS1 0, 10, 20 eV
    """
    # read feature tables
    ms2_dda_df = pd.read_csv('../../../../data/nist/dda_ms2/aligned_feature_table.tsv', sep='\t', low_memory=False)
    ms1_dda_df = pd.read_csv('../../../../data/nist/dda_ms1/aligned_feature_table.tsv', sep='\t', low_memory=False)
    ms1_0ev_df = pd.read_csv('../../../../data/nist/fullscan_0ev/aligned_feature_table.tsv', sep='\t', low_memory=False)
    ms1_10ev_df = pd.read_csv('../../../../data/nist/fullscan_10ev/aligned_feature_table.tsv', sep='\t',
                              low_memory=False)
    ms1_20ev_df = pd.read_csv('../../../../data/nist/fullscan_20ev/aligned_feature_table.tsv', sep='\t',
                              low_memory=False)

    ms1_dda_df = ms1_dda_df[ms1_dda_df['MS1_similarity'].notnull()].reset_index(drop=True)
    ms1_0ev_df = ms1_0ev_df[ms1_0ev_df['MS1_similarity'].notnull()].reset_index(drop=True)
    ms1_10ev_df = ms1_10ev_df[ms1_10ev_df['MS1_similarity'].notnull()].reset_index(drop=True)
    ms1_20ev_df = ms1_20ev_df[ms1_20ev_df['MS1_similarity'].notnull()].reset_index(drop=True)

    ms1_dda_df = ms1_dda_df[(ms1_dda_df['MS1_similarity'] >= score_threshold) &
                            (ms1_dda_df['MS1_spectral_usage'] >= usage_threshold)].reset_index(drop=True)
    ms1_0ev_df = ms1_0ev_df[(ms1_0ev_df['MS1_similarity'] >= score_threshold) &
                            (ms1_0ev_df['MS1_spectral_usage'] >= usage_threshold)].reset_index(drop=True)
    ms1_10ev_df = ms1_10ev_df[(ms1_10ev_df['MS1_similarity'] >= score_threshold) &
                              (ms1_10ev_df['MS1_spectral_usage'] >= usage_threshold)].reset_index(drop=True)
    ms1_20ev_df = ms1_20ev_df[(ms1_20ev_df['MS1_similarity'] >= score_threshold) &
                              (ms1_20ev_df['MS1_spectral_usage'] >= usage_threshold)].reset_index(drop=True)

    # align to DDA MS2
    ms2_dda_df['ms1_dda_inchikeys'] = [None] * ms2_dda_df.shape[0]
    ms2_dda_df['ms1_0ev_inchikeys'] = [None] * ms2_dda_df.shape[0]
    ms2_dda_df['ms1_10ev_inchikeys'] = [None] * ms2_dda_df.shape[0]
    ms2_dda_df['ms1_20ev_inchikeys'] = [None] * ms2_dda_df.shape[0]

    for df, col_name in zip([ms1_dda_df, ms1_0ev_df, ms1_10ev_df, ms1_20ev_df],
                            ['ms1_dda_inchikeys', 'ms1_0ev_inchikeys', 'ms1_10ev_inchikeys', 'ms1_20ev_inchikeys']):
        print(f'Aligning {col_name}...')

        df['used'] = [False] * df.shape[0]

        for i, row in ms2_dda_df.iterrows():
            prec_mz = row['m/z']
            rt = row['RT']

            # find in feature table
            mask = ((df['m/z'] - prec_mz).abs() <= 0.01) & ((df['RT'] - rt).abs() <= 0.1)
            if sum(mask) > 0:
                ms2_dda_df.at[i, col_name] = df[mask]['MS1_inchikey'].tolist()
                df.loc[mask, 'used'] = True

        print(f'{col_name}: {sum(df["used"])} annotations aligned')

        _df = ms2_dda_df[(ms2_dda_df['MS2'].notnull()) & (ms2_dda_df[col_name].notnull())]
        print(f'{col_name}: DDA MS2 collected, {len(_df)} annotations')

        _df = ms2_dda_df[(ms2_dda_df['MS2'].isnull()) & (ms2_dda_df[col_name].notnull())]
        print(f'{col_name}: DDA MS2 not collected, {len(_df)} annotations')

        print(f'{col_name}: {len(df) - sum(df["used"])} new annotations')

        # save
        out_name = f'data/{col_name.split("_inchi")[0]}_df.pkl'
        df.to_pickle(out_name)

    # save
    ms2_dda_df.to_csv('data/aligned_feature_table_all.tsv', sep='\t', index=False)
    ms2_dda_df.to_pickle('data/aligned_feature_table_all.pkl')


if __name__ == '__main__':
    print_annotation_stats()
    #
    align_feature_tables()



import pandas as pd
import os


def find_derep_tsv_files(folder):
    derep_tsv_files = []

    for root, dirs, files in os.walk(folder):
        for file in files:
            if file.endswith('_derep.tsv'):
                full_path = os.path.join(root, file)
                derep_tsv_files.append(full_path)

    return derep_tsv_files


def merge(folder, out_file):

    derep_files = find_derep_tsv_files(folder)

    print(f"Found {len(derep_files)} _derep.tsv files:")

    all_dfs = []
    for file in derep_files:
        dataset_name = file.split('/')[-2]
        this_df = pd.read_csv(file, sep='\t')
        this_df['dataset'] = dataset_name
        all_dfs.append(this_df)

    all_df = pd.concat(all_dfs)
    all_df['inchikey_14'] = all_df['inchikey'].str[:14]

    all_df.to_csv(f'{out_file}.tsv', sep='\t', index=False)

    # sort by matched_score
    all_df.sort_values('matched_score', ascending=False, inplace=True)

    # group by inchikey_14, keep first row
    all_grouped = all_df.groupby('inchikey_14')
    out_df = all_grouped.first().reset_index()
    # add a col for the number of unique datasets
    out_df['num_datasets'] = all_grouped['dataset'].nunique().reset_index()['dataset']

    out_df.to_csv(f'{out_file}_grouped.tsv', sep='\t', index=False)


if __name__ == '__main__':
    # merge('/Users/shipei/Documents/projects/ms1_id/imaging/hepatocytes', 'data/all_hepatocytes_derep')

    merge('/Users/shipei/Documents/projects/ms1_id/imaging/HeLa_NIH3T3', 'data/all_HeLa_NIH3T3_derep')
